"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DashTech = void 0;
const nas_logger_1 = require("nas-logger");
const dashjs_1 = require("dashjs");
const utils_1 = require("../utils");
class DashTech {
    url = '';
    player = null;
    headers = null;
    logger = new nas_logger_1.Logger('DashTech');
    ;
    eventHandler = null;
    is_live = false;
    autoplay = false;
    onLicenseError = null;
    videoElement = null;
    defaultHandler = (e) => {
        this.eventHandler(e);
    };
    ascertainStreamType = (e) => {
        if (e.data.type == 'dynamic') {
            this.is_live = true;
        }
        this.eventHandler(e);
    };
    playerErrorHandler = (e) => {
        this.logger.e(e.error.message);
        if (e.error.code == 111 || e.error == 'key_session' && this.onLicenseError()) {
            this.onLicenseError();
            return;
        }
        if (this.eventHandler) {
            this.eventHandler(e);
        }
        else {
            this.logger.w('eventHandler is undefined');
        }
        this.defaultHandler(e);
        this.destroy();
    };
    init(videoElement, url, autoplay, debug, eventHandler, headers, protData, onLicenseError) {
        this.logger.debug = debug;
        this.url = url;
        this.headers = headers;
        this.autoplay = autoplay;
        this.videoElement = videoElement;
        this.onLicenseError = onLicenseError;
        this.player = (0, dashjs_1.MediaPlayer)().create();
        this.eventHandler = eventHandler;
        this.attachHandlers();
        if (typeof this.player.setFastSwitchEnabled != 'undefined') {
            this.player.setFastSwitchEnabled(true);
        }
        if (typeof this.player.getDebug().setLogToBrowserConsole !== 'undefined') {
            this.player.getDebug().setLogToBrowserConsole(debug);
        }
        if (protData != null) {
            this.player.getProtectionController().setRobustnessLevel('SW_SECURE_CRYPTO');
            this.player.setProtectionData(protData);
        }
        var requestHeadersMod = (xhr) => {
            for (var header_name in this.headers) {
                xhr.setRequestHeader(header_name, this.headers[header_name]);
            }
            return xhr;
        };
        if (null != this.headers) {
            this.player.extend("RequestModifier", () => {
                return {
                    modifyRequestHeader: requestHeadersMod
                };
            }, true);
        }
        this.player.label = "dash";
        this.logger.d('initializing');
        let settings = {
            'errors': {
                'recoverAttempts': {
                    'mediaErrorDecode': 3
                }
            },
            'streaming': {
                'capabilities': {
                    'filterUnsupportedEssentialProperties': true,
                    'useMediaCapabilitiesApi': true
                }
            }
        };
        this.player.registerCustomCapabilitiesFilter((representation) => {
            let type = representation['mimeType'] + '; codecs="' + representation['codecs'] + '"';
            let canPlay = '' !== this.videoElement.canPlayType(type);
            console.log('Can play ' + type + ': ' + canPlay);
            return canPlay;
        });
        this.player.updateSettings(settings);
        this.player.initialize(this.videoElement, this.url, this.autoplay);
    }
    attachHandlers() {
        this.player.on(dashjs_1.MediaPlayer.events.METRIC_CHANGED, this.defaultHandler);
        this.player.on(dashjs_1.MediaPlayer.events.STREAM_INITIALIZED, this.defaultHandler);
        this.player.on(dashjs_1.MediaPlayer.events.MANIFEST_LOADED, this.ascertainStreamType);
        this.player.on(dashjs_1.MediaPlayer.events.ERROR, this.playerErrorHandler);
    }
    detachHandlers() {
        this.player.off(dashjs_1.MediaPlayer.events.METRIC_CHANGED, this.defaultHandler);
        this.player.off(dashjs_1.MediaPlayer.events.STREAM_INITIALIZED, this.defaultHandler);
        this.player.off(dashjs_1.MediaPlayer.events.MANIFEST_LOADED, this.ascertainStreamType);
        this.player.off(dashjs_1.MediaPlayer.events.ERROR, this.playerErrorHandler);
    }
    getPlayer() {
        return this.player;
    }
    isLive() {
        return this.is_live;
    }
    getAudioTracks() {
        var u = this.player.getTracksFor('audio');
        var audio_list = [];
        for (var i = 0; i < u.length; i++) {
            if ('' == u[i].lang) {
                u[i].lang = 'Audio ' + i;
            }
            if ('' == u[i].name) {
                u[i].name = 'Audio ' + i;
            }
            audio_list.push({
                lang: u[i].lang,
                name: u[i].name,
                index: u[i].index
            });
        }
        return audio_list;
    }
    setAudioTrack(index) {
        var tracks = this.player.getTracksFor('audio');
        for (var i = 0; i < tracks.length; i++) {
            if (tracks[i].index == index) {
                this.player.setCurrentTrack(tracks[i]);
                return;
            }
        }
    }
    getQualities() {
        var u = this.player.getBitrateInfoListFor("video");
        var bitrates = [];
        var b = { index: -1, bitrate: 0, bitrateStr: 'Auto', width: 0, height: 0 };
        bitrates.push(b);
        for (var i = 0; i < u.length; i++) {
            var b = { index: 0, bitrate: 0, bitrateStr: '0k', width: 0, height: 0 };
            b.index = u[i].qualityIndex;
            b.bitrate = u[i].bitrate;
            b.bitrateStr = utils_1.Utils.calcBitrateStr(u[i].bitrate);
            b.width = u[i].width;
            b.height = u[i].height;
            bitrates.push(b);
        }
        return bitrates;
    }
    setQuality(index) {
        index = parseInt(index);
        if (index == -1) {
            if (typeof this.player.setAutoSwitchQuality !== 'undefined') {
                this.player.setAutoSwitchQuality(true);
            }
            else if (typeof this.player.setAutoSwitchQualityFor !== 'undefined') {
                this.player.setAutoSwitchQualityFor('video', true);
            }
            else if (typeof this.player.updateSettings !== 'undefined') {
                this.player.updateSettings({
                    'streaming': {
                        'abr': {
                            'autoSwitchBitrate': {
                                'video': true
                            }
                        }
                    }
                });
            }
            return;
        }
        if (typeof this.player.setAutoSwitchQuality !== 'undefined') {
            this.player.setAutoSwitchQuality(false);
        }
        else if (typeof this.player.setAutoSwitchQualityFor !== 'undefined') {
            this.player.setAutoSwitchQualityFor('video', false);
        }
        else if (typeof this.player.updateSettings !== 'undefined') {
            this.player.updateSettings({
                'streaming': {
                    'abr': {
                        'autoSwitchBitrate': {
                            'video': false
                        }
                    }
                }
            });
        }
        this.player.setQualityFor("video", index);
    }
    getCurrentQuality() {
        if (!this.player) {
            return { index: 0, bitrate: 0, bitrateStr: '0k', width: 0, height: 0 };
        }
        var q = this.player.getQualityFor('video');
        var qualities = this.getQualities();
        for (var quality in qualities) {
            if (q == qualities[quality].index) {
                return qualities[quality];
            }
        }
        var b = { index: 0, bitrate: 0, bitrateStr: '0k', width: 0, height: 0 };
        b.index = q.qualityIndex;
        b.bitrate = q.bitrate;
        b.bitrateStr = Math.floor(q.bitrate / 1000) + 'k';
        b.width = q.width;
        b.height = q.height;
        return b;
    }
    setMaxQuality() {
        var qualities = this.getQualities();
        var quality = qualities[0];
        for (var i = 1; i < qualities.length; i++) {
            if (qualities[i].bitrate > quality.bitrate) {
                quality = qualities[i];
            }
        }
        this.setQuality(quality.index);
    }
    destroy() {
        if (this.player != null) {
            this.detachHandlers();
            this.logger.d("Dashjs destroy");
            this.player.reset();
            this.player = null;
        }
    }
}
exports.DashTech = DashTech;
